#include <config.h>
module config_profile_class
  use lattice_class
  implicit none
  private
  public :: config_profile_obj
  public :: read,save,new,delete,print,copy

  type config_profile_obj
    integer :: NTX,NTY,NTZ,NTT
    integer :: NDIMX,NDIMY,NDIMZ
    integer :: ipsite(NDIM-1)
    integer :: nodeid
    integer :: random_number_seed
    integer :: total_trajectory
    character(len=CHARLEN) :: cdate,ctime,czone,comment
    real(DP) :: plq
  end type

  interface new
    module procedure new_config_prof
  end interface

  interface delete
    module procedure delete_config_prof
  end interface

  interface copy
    module procedure copy_config_prof
  end interface
  
  interface read
    module procedure read_config_prof
  end interface

  interface save
    module procedure save_config_prof
  end interface

  interface print
    module procedure print_config_prof
  end interface

contains

subroutine new_config_prof(this)
  use lattice_class
  implicit none
  type(config_profile_obj), intent(inout) :: this

  this%NTT = NTT
  this%NTZ = NTZ
  this%NTY = NTY
  this%NTX = NTX

  this%NDIMZ = NDIMZ
  this%NDIMY = NDIMY
  this%NDIMX = NDIMX

  this%nodeid   = nodeid
  this%ipsite(:)= ipsite(:)

  this%comment = REPEAT(' ',LEN(this%comment))
  this%cdate   = REPEAT(' ',LEN(this%cdate))
  this%ctime   = REPEAT(' ',LEN(this%ctime))
  this%czone   = REPEAT(' ',LEN(this%czone))

  return
end subroutine

subroutine delete_config_prof(this)
  use lattice_class
  implicit none
  type(config_profile_obj), intent(inout) :: this
  return
end subroutine

subroutine copy_config_prof(this,that)
! copy data : that <= this
  implicit none
  type(config_profile_obj), intent(inout) :: this
  type(config_profile_obj), intent(inout) :: that

  that%NTX=this%NTX
  that%NTY=this%NTY
  that%NTZ=this%NTZ
  that%NTT=this%NTT
  that%NDIMX=this%NDIMX
  that%NDIMY=this%NDIMY
  that%NDIMZ=this%NDIMZ
  that%nodeid=this%nodeid
  that%ipsite(:)=this%ipsite(:)
  that%plq=this%plq
  that%cdate=this%cdate
  that%ctime=this%ctime
  that%czone=this%czone
  that%comment=this%comment
  that%random_number_seed=this%random_number_seed
  that%total_trajectory=this%total_trajectory
  
  return
end subroutine

subroutine read_config_prof(this,iout)
  implicit none
  type(config_profile_obj), intent(inout) :: this
  integer,                  intent(in)    :: iout

  read(iout) this%NTX,this%NTY,this%NTZ,this%NTT
  read(iout) this%NDIMX,this%NDIMY,this%NDIMZ
  read(iout) this%plq

!  read(iout) this%nodeid
!  read(iout) this%ipsite(1),this%ipsite(2),this%ipsite(3)
!  read(iout) this%cdate
!  read(iout) this%ctime
!  read(iout) this%czone
!  read(iout) this%comment
!  read(iout) this%random_number_seed
!  read(iout) this%total_trajectory

  return
end subroutine

subroutine save_config_prof(this,iout)
  implicit none
  type(config_profile_obj), intent(inout) :: this
  integer,                  intent(in)    :: iout
  character(len=CHARLEN) :: cdate,ctime,czone

  call DATE_AND_TIME(DATE=cdate,TIME=ctime,ZONE=czone)
  this%cdate = TRIM(ADJUSTL(cdate))
  this%ctime = TRIM(ADJUSTL(cdate))
  this%czone = TRIM(ADJUSTL(czone))

  write(iout) this%NTX,this%NTY,this%NTZ,this%NTT
  write(iout) this%NDIMX,this%NDIMY,this%NDIMZ
  write(iout) this%plq

!  write(iout) this%nodeid
!  write(iout) this%ipsite(1),this%ipsite(2),this%ipsite(3)
!  write(iout) this%plq
!  write(iout) this%cdate
!  write(iout) this%ctime
!  write(iout) this%czone
!  write(iout) this%comment
!  write(iout) this%random_number_seed
!  write(iout) this%total_trajectory
  
  return
end subroutine

subroutine print_config_prof(this)
  use lattice_class
  implicit none
  type(config_profile_obj), intent(in) :: this

  if (nodeid == 0) then
    write(*,'("      NTX, NTY, NTZ, NTT =",4I3)') this%NTX,   this%NTY,   this%NTZ, this%NTT
    write(*,'("     NDIMX, NDIMY, NDIMZ =",3I3)') this%NDIMX, this%NDIMY, this%NDIMZ
    write(*,'("                  NODEID =",I4)') this%nodeid
    write(*,'("      IPX, IPY, IPZ, IPT =",4I4)') this%ipsite(1), this%ipsite(2), this%ipsite(3)
    write(*,'("                     plq =",F20.16)') this%plq
    write(*,'("                   cdate =",A)') TRIM(this%cdate)
    write(*,'("                   ctime =",A)') TRIM(this%ctime)
    write(*,'("                   czone =",A)') TRIM(this%czone)
    write(*,'("                 comment =",A)') TRIM(this%comment)
    write(*,'("      random_number_seed =",I10)') this%random_number_seed
    write(*,'("        total_trajectory =",I10)') this%total_trajectory
  endif

  return
end subroutine

end module
