static __global__ void 
assign_mult_clover_term_kernel(      cu_wqf_eo_blk  *fye,
                               const int4           *blk_indx,
                               const cu_wqf_eo_blk  *ye,
                               const cu_clvf_eo_blk *fclinve)
////////////////////////////////////////////////////////////////////////////////
// Multiply inverse clover term : fye = Fee ye
//
//     fye : even/odd site fermion (ieo=0/1) (output)
//      ye : odd/even site fermion (ieo=0/1) (input)
//     ieo : operate even/odd site
//           ieo=0 for even site
//           ieo=1 for odd  site
// fclinve : even/odd site inverse clover term (ieo=0/1)
////////////////////////////////////////////////////////////////////////////////
{
  __align__(16) float  ytr[CLSP],yti[CLSP];
  int ithd = threadIdx.x;
#ifdef _SF
  int ibx  = blk_indx[blockIdx.x].x;
  int iby  = blk_indx[blockIdx.x].y;
  int ibz  = blk_indx[blockIdx.x].z;
#endif
  int iblk = blk_indx[blockIdx.x].w;
  int ieo = (*ye).ieo;

  (*fye).igx  = (*ye).igx;
  (*fye).ieo  = ieo;
  (*fye).ipeo = (*ye).ipeo;
  (*fye).ipsite.x = (*ye).ipsite.x;
  (*fye).ipsite.y = (*ye).ipsite.y;
  (*fye).ipsite.z = (*ye).ipsite.z;
  (*fye).ipsite.w = (*ye).ipsite.w;

  *(float4 *)(ytr+0) = (*ye).blk[iblk].thd[0].f[ithd];
  *(float4 *)(ytr+4) = (*ye).blk[iblk].thd[1].f[ithd];
  *(float4 *)(ytr+8) = (*ye).blk[iblk].thd[2].f[ithd];
  *(float4 *)(yti+0) = (*ye).blk[iblk].thd[3].f[ithd];
  *(float4 *)(yti+4) = (*ye).blk[iblk].thd[4].f[ithd];
  *(float4 *)(yti+8) = (*ye).blk[iblk].thd[5].f[ithd];

  cu_mult_clover_term(&((*fclinve).blk[iblk]),&ithd,ytr,yti);

#ifdef _SF
  {
//
//  Set SF boundary zero (Fortran: itt=1 (C: itt=0))
//
    int iz     = thd_z + ibz*_NTHZ;
    int iy     =         iby;
    int ix     =         ibx + (igx-1)*_NBX;
    int ieoxyz = (ipeo+ieo+(iz+1)+(iy+1)+(ix+1)+256)%2;
    int it     = (thd_t+1-ieoxyz)*2+ieoxyz-1;
    float flag = (float)(1-(_NTT-it)/_NTT);
#pragma unroll 12
    for (unsigned int i = 0; i < CLSP; i++) {
      ytr[i] = ytr[i] * flag;
      yti[i] = yti[i] * flag;
    } 
  }
#endif

  (*fye).blk[iblk].thd[0].f[ithd] = *(float4 *)(ytr+0);
  (*fye).blk[iblk].thd[1].f[ithd] = *(float4 *)(ytr+4);
  (*fye).blk[iblk].thd[2].f[ithd] = *(float4 *)(ytr+8);
  (*fye).blk[iblk].thd[3].f[ithd] = *(float4 *)(yti+0);
  (*fye).blk[iblk].thd[4].f[ithd] = *(float4 *)(yti+4);
  (*fye).blk[iblk].thd[5].f[ithd] = *(float4 *)(yti+8);
}

static void 
assign_mult_clover_term(      cu_wqf_eo *fye,
                        const cu_wqf_eo *ye,
                        const cu_clvf   *fclinv)
{
  int ieo = (*ye).ieo;
  (*fye).ieo  = ieo;
  (*fye).ipeo = (*ye).ipeo;
#pragma omp parallel for
  for (int igx=0;igx<_NGX;igx++) {
    switch_gpu_(&igx);
    assign_mult_clover_term_kernel<<<_NBLOCKS,_NTHREADS>>>((*fye).g[igx],phys_blk_d.g[igx],
                                                            (*ye).g[igx],
                                                (*fclinv).eo[ieo].g[igx]);
    CUT_CHECK_ERROR("assign_mult_clover_term kernel execution faild");
  }
}
