#include <cuda.h>
#include <omp.h>
#include <cassert>
#include <mm_malloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <cutil.h>

#include "cconfig.h"
#include "cgpu_config.h"
#include "cgpu_solver_config.h"
#include "omp_thread_gpu_dev_table.h"
#include "cuparam-size.h"

//
// Fortran Module Variables
//
#include "cufortran_vars.h"

static int g_num_use_gpus = 0;

extern "C" void 
cuda_bind_cpu_to_gpu_(int *num_total_gpus, int *num_use_gpus){
//
// Bind GPU to OpenMP-threads
//

  int num_gpus;

  CUDA_SAFE_CALL(cudaGetDeviceCount(&num_gpus));
  if (num_gpus < 1) {
    if (_nodeid_==0) 
      printf("There is no CUDA Device. Program Stops. MPI_RANK %5d \n",_nodeid_);
    exit(-1);
  }

  *num_total_gpus = num_gpus;
  g_num_use_gpus = *num_use_gpus;
  int cuda_version = CUDA_VERSION;

  if ( 0 == _nodeid_ ) {
    printf("%%==============================================================================\n");
    printf("%% GPUs are binded to CPU threads. CUDA version = %4d\n",cuda_version);
#ifdef _OPENMP
    printf("%% Number of Host OpenMP Threads : %d\n",omp_get_max_threads());
#else
    printf("%%                 Single Thread : %d\n",1);
#endif
    printf("%% Number of CUDA Devices        : %d\n",num_gpus);
  }

#pragma omp parallel for
  for (int iomp=0;iomp < g_num_use_gpus;iomp++) {
#ifdef _OPENMP
    int omp_thread_num  = omp_get_thread_num();
#else
    int omp_thread_num  = 0;
#endif
//    int cuda_device_num = igx % num_gpus;
    int cuda_device_num = devidx[iomp % g_num_use_gpus];
    if ( 0 == _nodeid_ )
      printf("%% OpenMP Thread = %d  Cuda Device = %d\n",omp_thread_num,cuda_device_num);
#ifdef _MULTIGPU_
    CUDA_SAFE_CALL(cudaSetDevice(cuda_device_num));
#endif
    
    CUDA_SAFE_CALL(cudaThreadSetCacheConfig(cudaFuncCachePreferL1));

  }  
  if ( 0 == _nodeid_ )
    printf("%%==============================================================================\n");

  fflush(stdout);
}

extern "C" void 
cuda_unbind_cpu_to_gpu_(){
//
// Unbind GPUs and OpenMP threads
//
#pragma omp parallel for
  for (int iomp=0;iomp < g_num_use_gpus;iomp++) {
#if CUDA_VERSION >= 4000
    int cuda_device_num = devidx[(iomp) % g_num_use_gpus];
#ifdef _MULTIGPU_
    CUDA_SAFE_CALL(cudaSetDevice(cuda_device_num));
    CUDA_SAFE_CALL(cudaThreadExit());
#endif
#endif
  }
}

extern "C" void 
switch_gpu_(const int *iomp){
#if CUDA_VERSION >= 4000
    int cuda_device_num = devidx[(*iomp) % g_num_use_gpus];
#ifdef _MULTIGPU_
    CUDA_SAFE_CALL(cudaSetDevice(cuda_device_num));
#endif
#endif
}
