#include <config.h>

module read_dumpy_class
!
!
  use lattice_class
  use timer_class
  use field_fermion_class
  implicit none
  private

  public :: read_dumpy

contains

subroutine read_dumpy(fname,y)
  use file_tools_class
  implicit none
  character(len=*),      intent(in)    :: fname
  type(field_quark_wg),  intent(inout) :: y
  complex(DP), allocatable :: yy(:,:,:,:,:,:)

  integer :: ipx,ipy,ipz
  integer :: ix,iy,iz,it,ieo,ieoxyz,ith
  integer :: iout
  integer :: itx,ity,itz,itt,ic,is
  real(DP) :: rtmp

  call new(y)
  allocate(yy(COL,SPIN,NTX,NTY,NTZ,NTT))

  iout = search_free_file_unit()
  open(iout,file=TRIM(ADJUSTL(fname)),access="stream",form="unformatted",status="old")
  read(iout)yy
  close(iout)

  !-----------------------------
  ! splitting, reordering
  !              and storing
  !-----------------------------
  ipx = ipsite(1)
  ipy = ipsite(2)
  ipz = ipsite(3)
!  do ipz=0,NDIMZ-1
!  do ipy=0,NDIMY-1
!  do ipx=0,NDIMX-1

!$OMP PARALLEL DO PRIVATE(ix,iy,iz,it,itx,ity,itz,itt,ith,ieo,ic,is)
    do ix=1,NX
    do iy=1,NY
    do iz=1,NZ
    do it=1,NT
      itx = ix + ipx*NX
      ity = iy + ipy*NY
      itz = iz + ipz*NZ
      itt = it
      ith = it/2
      ieo = mod(itt+itz+ity+itx,2)
      do is=1,SPIN
      do ic=1,COL
        y%eo(ieo)%s(ith,iz,iy,ix)%y(ic,is)  &
 &                         = yy(ic,is,itx,ity,itz,itt)
      enddo
      enddo
    enddo
    enddo
    enddo
    enddo
!  enddo
!  enddo
!  enddo

!  if (0 == nodeid) then
!    write(*,'(" |yy|^2 = ",E24.15)')rtmp
!  endif

  deallocate(yy)
  call copy_boundary(y)

  rtmp = abs2(y)
  if (0 == nodeid) then
    write(*,'(" |y|^2 = ",E24.15)')rtmp
  endif

  return
end subroutine

end module
