#include <config.h>

!
!= Hadron measurement class
!
!== Version
!
! $Id: hadron_class.F90,v 1.4 2011/10/31 08:08:38 ishikawa Exp $
!
module hadron_class
!
! This contains hadron propagators with up/down(degenerate) and strange quarks.
!
!
!
  use lattice_class
  use error_class
  use timer_class
  use comlib
  use field_gauge_class
  use config_profile_class
  use wavefunc_class
  use fftw3_class
  use file_tools_class
#ifdef _CLOVER
  use quark_clover_class
#else
  use quark_wilson_class
#endif
  use quark_prop_class
!  use wilson_quark_parameter_class
  implicit none
  private

  public :: new,delete
  public :: hadrons_obj
  public :: meson_prop_obj
  public :: meson_mom_prop_obj
  public :: baryon_prop_obj
  public :: get_prop, save
  public :: NUM_MESON_OP
  public :: NUM_MESON_OP_MOM,NUM_MOM
  public :: NUM_MESON_OP_LOCAL
  public :: NUM_BARYON_OP
  public :: NUM_DELTA_BARYON
  public :: NUM_OCTET_BARYON
  public :: HADRON_OUTPUT_FORMAT
  public :: print_config_prof


  integer, save :: io_unit=100

  integer, parameter :: NUM_DELTA_BARYON=4
  integer, parameter :: NUM_OCTET_BARYON=2
  integer, parameter :: NUM_BARYON_OP=NUM_DELTA_BARYON*2+4*NUM_OCTET_BARYON

  integer, parameter :: NUM_MESON_OP=16+2+9+3
  integer, parameter :: NUM_MESON_OP_LOCAL=18
  integer, parameter :: NUM_MESON_OP_MOM=4
  integer, parameter :: NUM_MOM=19
  integer, parameter :: NSPACE=NX*NY*NZ
  complex(DP), save  :: zphi(NSPACE,NUM_MOM)   ! mementum phase table
  integer,     save  :: ig5a(SPIN,0:15),is5a(SPIN,0:15)     ! current operator gamma matrix index
  integer,     save  ::  i5a(0:15),ia5(0:15)                ! current operator gamma matrix phase
  complex(DP), save  :: zj(0:3)                             ! complex number
  data zj /(1.0_DP,0.0_DP),(0.0_DP,1.0_DP),(-1.0_DP,0.0_DP),(0.0_DP,-1.0_DP)/
  data ig5a / 2,3,0,1,   &
 &            0,1,2,3,   &

 &            1,0,3,2,   &
 &            1,0,3,2,   &
 &            0,1,2,3,   &
 &            2,3,0,1,   &

 &            3,2,1,0,   &
 &            3,2,1,0,   &
 &            2,3,0,1,   &
 &            0,1,2,3,   &

 &            2,3,0,1,   &
 &            3,2,1,0,   &
 &            1,0,3,2,   &
 &            3,2,1,0,   &
 &            1,0,3,2,   &
 &            0,1,2,3 /
  data is5a / 1, 1, 1, 1,   &
 &            1, 1, 1, 1,   &
 &            1, 1,-1,-1,   &
 &            1,-1,-1, 1,   &
 &            1,-1,-1, 1,   &
 &            1, 1,-1,-1,   &
 &            1, 1,-1,-1,   &
 &            1,-1,-1, 1,   &
 &            1,-1,-1, 1,   &
 &            1, 1,-1,-1,   &
 &            1,-1, 1,-1,   &
 &            1,-1, 1,-1,   &
 &            1, 1, 1, 1,   &
 &            1, 1, 1, 1,   &
 &            1,-1, 1,-1,   &
 &            1,-1, 1,-1 /
  data i5a /   2,0,  &
 &             3,2,3,0,  &
 &             0,3,0,1,  &
 &             0,1,0,0,3,0 /
  data ia5 /   2,0,  &
 &             1,0,1,2,  &
 &             2,1,2,3,  &
 &             0,1,0,0,3,0 /
  integer, save :: color(0:35,2),icG(0:3,0:2),scG(0:3,0:2)
  integer, save :: ic5(0:3),sc5(0:3)
  data color /4,5,3,7,8,6,1,2,0,8,6,7,2,0,1,5,3,4,  &
 &            7,8,6,1,2,0,4,5,3,5,3,4,8,6,7,2,0,1,  &
 &            8,6,7,2,0,1,5,3,4,4,5,3,7,8,6,1,2,0,  &
 &            5,3,4,8,6,7,2,0,1,7,8,6,1,2,0,4,5,3/
  data icG /0,1,2,3,  &
 &          0,1,2,3,  &
 &          1,0,3,2/
  data scG /-1, 0, 1, 0,  &
 &           0, 1, 0,-1,  &
 &           1, 1,-1,-1/
  data ic5 /1,0,3,2/
  data sc5 /1,-1,1,-1/

!  character(len=22), parameter :: HADRON_OUTPUT_FORMAT="(I3,2X,E15.8,2X,E15.8)"
  character(len=24), parameter :: HADRON_OUTPUT_FORMAT="(I3,2X,E24.15,2X,E24.15)"

  type hadrons_obj
    integer :: idummy(4)
  end type

  type baryon_prop_obj
  !  
  ! Baryon propagators  
  !  
    complex(DP) :: prop(NTT,0:NUM_BARYON_OP-1)
    type(qprop_params)     :: quark_mass(3)
    character(len=CHARLEN) :: fname="hpropB"
    character(len=CHARLEN) :: path
    type(timer) :: run_time
    type(src_wavefunc_obj) :: wf_sous(3)
    type(snk_wavefunc_obj) :: wf_sink(3)
    type(config_profile_obj), pointer :: config_prof => NULL()
    integer :: smear_sous(3)
    integer :: smear_sink(3)
    integer :: iout
    integer :: itraj
  end type

  type meson_prop_obj
  !  
  ! Meson propagators
  !  
    complex(DP) :: prop(NTT,0:NUM_MESON_OP-1)
    type(qprop_params)     :: quark_mass(2)
    character(len=CHARLEN) :: fname="hpropM"
    character(len=CHARLEN) :: path
    type(timer) :: run_time
    type(src_wavefunc_obj) :: wf_sous(2)
    type(snk_wavefunc_obj) :: wf_sink(2)
    type(config_profile_obj), pointer :: config_prof => NULL()
    integer :: smear_sous(2)
    integer :: smear_sink(2)
    integer :: iout
    integer :: itraj
  end type

  type meson_mom_prop_obj
  !  
  ! Meson propagators with finite momentum
  !  
    complex(DP) :: prop(NTT,1:NUM_MESON_OP_MOM,1:NUM_MOM)
    type(qprop_params)     :: quark_mass(2)
    character(len=CHARLEN) :: fname="hpropMP"
    character(len=CHARLEN) :: path
    type(timer) :: run_time
    type(src_wavefunc_obj) :: wf_sous(2)
    type(snk_wavefunc_obj) :: wf_sink(2)
    type(config_profile_obj), pointer :: config_prof => NULL()
    integer :: smear_sous(2)
    integer :: smear_sink(2)
    integer :: iout
    integer :: itraj
  end type

  interface new
    module procedure  new_hadron
    module procedure  new_meson
    module procedure  new_meson_mom
    module procedure  new_baryon
  end interface

  interface delete
    module procedure  delete_meson
    module procedure  delete_meson_mom
    module procedure  delete_baryon
  end interface

  interface get_prop
    module procedure  get_meson_prop
    module procedure  get_meson_prop_lop
    module procedure  get_meson_mom_prop
    module procedure  get_baryon1_prop
    module procedure  get_baryon2C_prop
  end interface

  interface save
    module procedure  save_meson_prop
    module procedure  save_meson_mom_prop
    module procedure  save_baryon_prop
  end interface

contains

subroutine print_config_prof(iout,config_prof)
  implicit none
  integer, intent(in):: iout
  type(config_profile_obj), intent(in) :: config_prof
  integer :: iqk
  if (nodeid==0) then
    write(iout,'("begin config")')
    write(iout,'("  lattice_size(x,y,z,t) = ",4I3)')  &
 &                                   config_prof%NTX, &
 &                                   config_prof%NTY, &
 &                                   config_prof%NTZ, &
 &                                   config_prof%NTT
    write(iout,'("    PLQ = ",F24.15)')        config_prof%plq
    write(iout,'(" config_number = ",I10)')    config_prof%total_trajectory
    write(iout,'("         CDATE = ",A)') TRIM(config_prof%cdate)
    write(iout,'("         CTIME = ",A)') TRIM(config_prof%ctime)
    write(iout,'("         CZONE = ",A)') TRIM(config_prof%czone)
    write(iout,'("       COMMENT = ",A)') TRIM(config_prof%comment)
    write(iout,'("end config")')
  endif
  return
end subroutine

subroutine new_hadron(this)
!
! Initialize for Hadron measurement
!
  implicit none
  type(hadrons_obj), intent(inout) :: this

  call set_momentum_table
  this%idummy(:)=1

  return
end subroutine

subroutine new_meson(this)
!
! Initialize for meson propagators
!
  implicit none
  type(meson_prop_obj), intent(inout) :: this
  character(len=CHARLEN) :: fname, str
  integer :: err
  logical :: flag

!  if (nodeid==0) write(*,*)"meson_prop_obj:",sizeof(this),mod(sizeof(this),16)
  call new(this%run_time)
  this%prop = Z0

  this%iout = search_free_file_unit()

#ifndef _singlePU
  call comlib_bcast(this%iout,0)
#endif

  write(fname,'(I6.6)')this%itraj
  fname=TRIM(this%path)//'/'//TRIM(this%fname)//'.'//TRIM(fname)
  err = 0
  if (nodeid==0) &
    open(unit=this%iout,file=TRIM(fname),form='formatted',status='new',iostat=err)
#ifndef _singlePU
  call comlib_bcast(err,0)
#endif
  if (err/=0) then
    write(str,'(" Error: file open, iostat=",I3," file=",A," at ",A,1X,I5)')err,TRIM(fname),__FILE__,__LINE__
    call error_stop(str)
  endif
  call print_config_prof(this%iout,this%config_prof)

  return
end subroutine

subroutine new_meson_mom(this)
!
! Initialize for meson propagators with finite momentum
!
  implicit none
  type(meson_mom_prop_obj), intent(inout) :: this
  character(len=CHARLEN) :: fname,str
  integer :: err
  logical :: flag

!  if (nodeid==0) write(*,*)"meson_mom_prop_obj:",sizeof(this),mod(sizeof(this),16)
  call new(this%run_time)
  this%prop = Z0

  this%iout = search_free_file_unit()

#ifndef _singlePU
  call comlib_bcast(this%iout,0)
#endif

  write(fname,'(I6.6)')this%itraj
  fname=TRIM(this%path)//'/'//TRIM(this%fname)//'.'//TRIM(fname)
  if (nodeid==0) &
    open(unit=this%iout,file=TRIM(fname),form='formatted',status='new',iostat=err)
#ifndef _singlePU
  call comlib_bcast(err,0)
#endif
  if (err/=0) then
    write(str,'(" Error: file open, iostat=",I3," file=",A," at ",A,1X,I5)')err,TRIM(fname),__FILE__,__LINE__
    call error_stop(str)
  endif
  call print_config_prof(this%iout,this%config_prof)

  return
end subroutine

subroutine new_baryon(this)
!
! Initialize for baryon propagators
!
  implicit none
  type(baryon_prop_obj), intent(inout) :: this
  character(len=CHARLEN) :: fname,str
  integer :: err
  logical :: flag

!  if (nodeid==0) write(*,*)"baryon_mom_prop_obj:",sizeof(this),mod(sizeof(this),16)
  call new(this%run_time)
  this%prop = z0

  this%iout = search_free_file_unit()

#ifndef _singlePU
  call comlib_bcast(this%iout,0)
#endif

  write(fname,'(I6.6)')this%itraj
  fname=TRIM(this%path)//'/'//TRIM(this%fname)//'.'//TRIM(fname)
  if (nodeid==0) &
    open(unit=this%iout,file=TRIM(fname),form='formatted',status='new',iostat=err)
#ifndef _singlePU
  call comlib_bcast(err,0)
#endif
  if (err/=0) then
    write(str,'(" Error: file open, iostat=",I3," file=",A," at ",A,1X,I5)')err,TRIM(fname),__FILE__,__LINE__
    call error_stop(str)
  endif
  call print_config_prof(this%iout,this%config_prof)

  return
end subroutine

subroutine delete_meson(this)
  implicit none
  type(meson_prop_obj), intent(inout) :: this

  if (nodeid==0) close(unit=this%iout)

  return
end subroutine

subroutine delete_meson_mom(this)
  implicit none
  type(meson_mom_prop_obj), intent(inout) :: this

  if (nodeid==0) close(unit=this%iout)

  return
end subroutine

subroutine delete_baryon(this)
  implicit none
  type(baryon_prop_obj), intent(inout) :: this

  if (nodeid==0) close(unit=this%iout)

  return
end subroutine

subroutine save_meson_prop(this,iflv)
  implicit none
  type(meson_prop_obj), intent(inout) :: this
  real(DP) :: kappa(2)
  integer, intent(in) :: iflv
  integer :: itt,iop,iqk

  if (nodeid==0) then
    write(this%iout,'("begin meson",I3)') iflv

    do iqk=1,2
      write(this%iout,'("begin quark_val ",I1)')iqk
      select type(p => this%quark_mass(iqk)%mass)
#ifdef _CLOVER
      type is (quark_clover)
#else
      type is (quark_wilson)
#endif
        kappa(iqk) = get_kappa(p)
        write(this%iout,'(" kappa_val = ",F11.8)') kappa(iqk)
#ifdef _CLOVER
        write(this%iout,'("   csw_val = ",F11.8)')   get_csw(p)
#else
        write(this%iout,'("   csw_val = ",F11.8)')   0.0_DP
#endif

      end select
      write(this%iout,'(" source_center(x,y,z,t) = ",4I3)')  &
 &                                        this%wf_sous(iqk)%itx0,  &
 &                                        this%wf_sous(iqk)%ity0,  &
 &                                        this%wf_sous(iqk)%itz0,  &
 &                                        this%wf_sous(iqk)%itt0
      write(this%iout,'(" smear_source = ",I3)') this%smear_sous(iqk)
      write(this%iout,'(" smear_sink   = ",I3)') this%smear_sink(iqk)
      write(this%iout,'("end quark_val ",I1)')iqk
    enddo

    write(this%iout,'(I3,2X,2F15.8,2X,2I3)')iflv, &
 &                      kappa(1), kappa(2),       &
 &                      this%smear_sous(1),       &
 &                      this%smear_sous(2)

    do iop=0,NUM_MESON_OP-1
      write(this%iout,'("op. ",I3," begin")')iop
      do itt=1,NTT
        write(this%iout,HADRON_OUTPUT_FORMAT)itt-1,this%prop(itt,iop)
      enddo
      write(this%iout,'("op. ",I3," end")')iop
    enddo

    write(this%iout,'("end meson",I3)') iflv
  endif

  return
end subroutine

subroutine save_meson_mom_prop(this,iflv)
  implicit none
  type(meson_mom_prop_obj), intent(inout) :: this
  real(DP) :: kappa(2)
  integer, intent(in) :: iflv
  integer :: itt,iop,imom,iqk

  if (nodeid==0) then
    write(this%iout,'("begin meson",I3)') iflv

    do iqk=1,2
      write(this%iout,'("begin quark_val ",I1)')iqk

      select type(p => this%quark_mass(iqk)%mass)
#ifdef _CLOVER
      type is (quark_clover)
#else
      type is (quark_wilson)
#endif

        kappa(iqk) = get_kappa(p)
        write(this%iout,'(" kappa_val = ",F11.8)') kappa(iqk)
#ifdef _CLOVER
        write(this%iout,'("   csw_val = ",F11.8)')   get_csw(p)
#else
        write(this%iout,'("   csw_val = ",F11.8)')   0.0_DP
#endif
      end select

      write(this%iout,'(" source_center(x,y,z,t) = ",4I3)')  &
 &                                        this%wf_sous(iqk)%itx0,  &
 &                                        this%wf_sous(iqk)%ity0,  &
 &                                        this%wf_sous(iqk)%itz0,  &
 &                                        this%wf_sous(iqk)%itt0
      write(this%iout,'(" smear_source = ",I3)') this%smear_sous(iqk)
      write(this%iout,'(" smear_sink   = ",I3)') this%smear_sink(iqk)
      write(this%iout,'("end quark_val ",I1)')iqk
    enddo

    write(this%iout,'(I3,2X,2F15.8,2X,2I3)')iflv, &
 &                      kappa(1), kappa(2),       &
 &                      this%smear_sous(1),       &
 &                      this%smear_sous(2)

    do iop=1,NUM_MESON_OP_MOM
      write(this%iout,'("op. ",I3," begin")')iop
      do imom=1,NUM_MOM
        write(this%iout,'("%momentum: ",I3," start")')imom
        do itt=1,NTT
          write(this%iout,HADRON_OUTPUT_FORMAT)itt-1,this%prop(itt,iop,imom)
        enddo
        write(this%iout,'("%momentum: ",I3," finish")')imom
      enddo
      write(this%iout,'("op. ",I3," end")')iop
    enddo

    write(this%iout,'("end meson",I3)') iflv
  endif

  return
end subroutine

subroutine save_baryon_prop(this,iflv)
  implicit none
  type(baryon_prop_obj), intent(inout) :: this
  real(DP) :: kappa(3)
  integer, intent(in) :: iflv
  integer :: itt,iop,iqk

  if (nodeid==0) then
    write(this%iout,'("begin baryon",I3)') iflv

    do iqk=1,3
      write(this%iout,'("begin quark_val ",I1)')iqk

      select type(p => this%quark_mass(iqk)%mass)
#ifdef _CLOVER
      type is (quark_clover)
#else
      type is (quark_wilson)
#endif

        kappa(iqk) = get_kappa(p)

        write(this%iout,'(" kappa_val = ",F11.8)') kappa(iqk)
#ifdef _CLOVER
        write(this%iout,'("   csw_val = ",F11.8)')   get_csw(p)
#else
        write(this%iout,'("   csw_val = ",F11.8)')   0.0_DP
#endif
      end select

      write(this%iout,'(" source_center(x,y,z,t) = ",4I3)')  &
 &                                        this%wf_sous(iqk)%itx0,  &
 &                                        this%wf_sous(iqk)%ity0,  &
 &                                        this%wf_sous(iqk)%itz0,  &
 &                                        this%wf_sous(iqk)%itt0
      write(this%iout,'(" smear_source = ",I3)') this%smear_sous(iqk)
      write(this%iout,'(" smear_sink   = ",I3)') this%smear_sink(iqk)
      write(this%iout,'("end quark_val ",I1)')iqk
    enddo

    write(this%iout,'(I3,2X,3F15.8,3X,3I3)')iflv, &
 &                      kappa(1), &
 &                      kappa(2), &
 &                      kappa(3), &
 &                      this%smear_sous(1),       &
 &                      this%smear_sous(2),       &
 &                      this%smear_sous(3)

    do iop=0,NUM_BARYON_OP-1
      write(this%iout,'("op. ",I3," begin")')iop
      do itt=1,NTT
        write(this%iout,HADRON_OUTPUT_FORMAT)itt-1,this%prop(itt,iop)
      enddo
      write(this%iout,'("op. ",I3," end")')iop
    enddo

    write(this%iout,'("end baryon",I3)') iflv
  endif

  return
end subroutine

#include "meson_localop.h90"
#include "meson_momentum.h90"
#include "meson_nonlocalop.h90"
#include "set_momentum_table.h90"
#include "baryon.h90"

end module
